local Blob = require('Blob')

local function _setQuads(particlesystem, pfx, imgW, imgH)
	if pfx.framesX * pfx.framesY > 1 then
		local quads = {}
		local qw, qh = math.floor(.5 + imgW / pfx.framesX), math.floor(.5 + imgH / pfx.framesY)

		for y = 1, pfx.framesY do
			for x = 1, pfx.framesX do
				local q = love.graphics.newQuad((x - 1) * qw, (y - 1) * qh, qw, qh, imgW, imgH)
				table.insert(quads, q)
			end
		end

		particlesystem:setQuads(unpack(quads))
	end
	return particlesystem
end

local function _newPS(data)
	local ps = love.graphics.newParticleSystem(love.graphics.newImage(data.imageData), data.count)
	local imgW, imgH = ps:getTexture():getDimensions()

	ps:setEmissionRate(data.emissionRate)
	ps:setEmitterLifetime(data.emitterLifeTime)
	ps:setParticleLifetime(data.particleLifeTime.min, data.particleLifeTime.max)
	ps:setSizes(unpack(data.sizes))
	ps:setSizeVariation(data.sizeVariation)
	ps:setInsertMode(data.insertMode)
	ps:setColors(unpack(data.colors))
	ps:setAreaSpread(data.areaSpread.distribution, data.areaSpread.dx, data.areaSpread.dy)
	ps:setDirection(data.direction)
	ps:setSpeed(data.speed.min, data.speed.max)
	ps:setLinearAcceleration(data.laccel.xmin, data.laccel.ymin, data.laccel.xmax, data.laccel.ymax)
	ps:setRadialAcceleration(data.raccel.min, data.raccel.max)
	ps:setTangentialAcceleration(data.taccel.min, data.taccel.max)
	ps:setLinearDamping(data.damping.min, data.damping.max)
	ps:setRotation(data.rotation.min, data.rotation.max)
	ps:setOffset(data.pivot.x * (imgW / (data.framesX or 1)), data.pivot.y * (imgH / (data.framesY or 1)))
	ps:setRelativeRotation(data.relativeRotation)
	ps:setSpin(data.spin.min, data.spin.max)
	ps:setSpinVariation(data.spinVariation)

	return _setQuads(ps, data, imgW, imgH)
end

local function loadPFX(filename)
	local filedata = love.filesystem.newFileData(filename)
	local blob = Blob(filedata:getString(), nil, { source_endianess = 'le' })
	local version = blob:readU32()
	assert(version == 0x01010102, "Invalid file or unknown version")
	local data = {
		count = blob:readU32(),
		framesX = blob:readU8(),
		framesY = blob:readU8(),
		emissionRate = blob:readNumber(),
		emitterLifeTime = blob:readNumber(),
		particleLifeTime = blob:readTable(),
		sizes = blob:readTable(),
		sizeVariation = blob:readNumber(),
		insertMode = blob:readString(),
		colors = blob:readTable(),
		areaSpread = blob:readTable(),
		direction = blob:readNumber(),
		speed = blob:readTable(),
		laccel = blob:readTable(),
		raccel = blob:readTable(),
		taccel = blob:readTable(),
		damping = blob:readTable(),
		rotation = blob:readTable(),
		pivot = blob:readTable(),
		relativeRotation = blob:readBool(),
		spin = blob:readTable(),
		spinVariation = blob:readNumber(),
		blendMode = blob:readString(),
		movement = blob:readString(),
		imageData = love.image.newImageData(love.filesystem.newFileData(blob:readString(), 'image.png'))
	}

	return _newPS(data)
end

return loadPFX